# -*- coding:utf-8 -*-
# Author：hankcs
# Date: 2018-03-18 20:05
import errno
import glob
import json
import os
import re
import sys
import urllib.request
import zipfile

from shutil import copyfile

STATIC_ROOT = os.path.dirname(os.path.realpath(__file__))
PATH_CONFIG = os.path.join(STATIC_ROOT, 'hanlp.properties')


def eprint(*args, **kwargs):
    print(*args, file=sys.stderr, **kwargs)


def remove_file(filename):
    try:
        os.remove(filename)
    except OSError as e:  # this would be "except OSError, e:" before Python 2.6
        if e.errno != errno.ENOENT:  # errno.ENOENT = no such file or directory
            raise  # re-raise exception if a different error occurred


def hanlp_latest_version():
    return hanlp_releases()[0]


def hanlp_releases():
    content = urllib.request.urlopen("http://api.github.com/repos/hankcs/HanLP/releases").read()
    content = json.loads(content)
    meta = []
    for r in content:
        jar_version = r['tag_name']
        if jar_version.startswith('v'):
            jar_version = jar_version[1:]
        m = re.search(r'\[(data-for-.*?\.zip)\]\((.*?)\)', r['body'])
        data_version, data_url = None, None
        if m and len(m.groups()) == 2:
            data_version = m.group(1)[len('data-for-'):-len('.zip')]
            data_url = m.group(2)
        meta.append((jar_version, data_version, data_url))

    return meta


def hanlp_installed_jar_versions():
    # print(glob.glob(os.path.join(STATIC_ROOT, 'hanlp-portable-{}.jar'.format(version))))
    # if not version:
    #     pass
    # if os.path.exists(version):
    #     pass
    versions = []
    for jar in glob.glob(os.path.join(STATIC_ROOT, 'hanlp-portable-*.jar')):
        versions.append(os.path.basename(jar)[len('hanlp-portable-'):-len('.jar')])

    return sorted(versions, reverse=True)


def hanlp_installed_data_path():
    root = read_config()
    if os.path.isdir(root):
        return root

    return None


def download(url, path):
    opener = urllib.request.build_opener()
    opener.addheaders = [('User-agent',
                          'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/65.0.3325.162 Safari/537.36')]
    urllib.request.install_opener(opener)
    eprint('Downloading {} to {}'.format(url, path))
    tmp_path = '{}.downloading'.format(path)
    try:
        urllib.request.urlretrieve(url, tmp_path)
    except:
        try:
            os.system('wget {} -O {}'.format(url, path))
        except:
            eprint('Failed to download {}'.format(url))
            return False
    os.rename(tmp_path, path)
    return True


def install_hanlp_jar(version=None):
    if version is None:
        # version = hanlp_latest_version()
        version = '1.6.0'
    url = 'http://search.maven.org/remotecontent?filepath=com/hankcs/hanlp/portable-{}/hanlp-portable-{}.jar'.format(
        version, version)
    download(url, hanlp_jar_path(version))


def update_hanlp_jar():
    if hanlp_latest_version()[0] in hanlp_installed_jar_versions():
        return
    install_hanlp_jar()
    uninstall_hanlp_jar()


def install_hanlp_data(version):
    for jar_version, data_version, data_url in hanlp_releases():
        if jar_version == version:
            data_zip = 'data-for-{}.zip'.format(data_version)
            data_zip = os.path.join(STATIC_ROOT, data_zip)
            download(data_url, os.path.join(STATIC_ROOT, data_zip))
            with zipfile.ZipFile(data_zip, "r") as zip_ref:
                zip_ref.extractall(STATIC_ROOT)
            os.remove(data_zip)
            write_config(root=STATIC_ROOT)
            break


def write_config(root=None):
    content = []
    with open(PATH_CONFIG) as f:
        for line in f:
            if root:
                if line.startswith('root'):
                    line = 'root={}{}'.format(root, os.linesep)
            content.append(line)
    with open(PATH_CONFIG, 'w') as f:
        f.writelines(content)


def read_config():
    root = None
    if not os.path.isfile(PATH_CONFIG):
        copyfile(PATH_CONFIG + '.in', PATH_CONFIG)
    with open(PATH_CONFIG) as f:
        for line in f:
            if line.startswith('root'):
                root = line.strip().split('=')[1]
    return root


def hanlp_jar_path(version):
    return os.path.join(STATIC_ROOT, 'hanlp-portable-{}.jar'.format(version))


def uninstall_hanlp_jar(version='old'):
    if version == 'old':
        vs = hanlp_installed_jar_versions()
        if len(vs) > 1:
            for v in vs[:-1]:
                remove_file(hanlp_jar_path(v))
    else:
        remove_file(hanlp_jar_path(version))


if not hanlp_installed_jar_versions():
    install_hanlp_jar()

if not hanlp_installed_data_path():
    install_hanlp_data(hanlp_installed_jar_versions()[0])

HANLP_JAR_PATH = hanlp_jar_path(hanlp_installed_jar_versions()[0])
HANLP_DATA_PATH = hanlp_installed_data_path()

# install_hanlp_jar('1.5.4')
# print(hanlp_releases())
# print(hanlp_installed('1.6.0'))
# print(hanlp_installed_jar_versions())
# uninstall_hanlp_jar()
# update_hanlp_jar()
# print(_HANLP_JAR_PATH)
# write_config(data_version='1.6.0')
# print(hanlp_installed_data_versions())
# download('http://storage.live.com/items/D4A741A579C555F7!65703:/data-for-1.6.0.zip', 'tmp')
